# -*- coding: iso_8859_1 -*-

from aspylib import astro, photometry, conf, astrometry
import numpy as np
import time, os, datetime
import matplotlib.pyplot as plt
import pp
from scipy import optimize

#------------------------------------------------------------------------------------
#		Input parameters (basic)
#------------------------------------------------------------------------------------

#--- input frames ---
Folder = u"C:\\images\\"
Imagelist = [Folder + "images-calibrated-" + str(i+1) + ".fit" for i in range(100)]

#--- nb of stars ---
nbdisk = 200			#nb of stars for calculating disk positions on each frame
nbref = 20			#nb of reference stars

#--- disk size for photometry ---
sizes = [[2,8,24],[3,10,24],[4.5,10,24]]

#--- output filenames ---
generic_name = Folder + 'target_'

#------------------------------------------------------------------------------------
#		Input parameters (advanced)
#------------------------------------------------------------------------------------

#--- semi-size of square areas used for star max detection and star fitting ---
psf_fit_area = 8			#typically a few times (2-3) the star FWHM.
max_detection_area = 2		#max_detection_area = 2 is recommended. Can be SLIGHTLY increased for very large stars (FWHM >5 pixels).

#--- threshold distance for star matching between reference positions and detected positions on new image ---
thdist = 20

#--- names of output files ---
outputname1 = generic_name + "01_check_images.txt"
outputname2 = generic_name + "02_image_shifts.txt"
outputname3 = generic_name + "03_magnitudes.txt"

#####################################################################################
#	SCRIPT STARTS HERE
#####################################################################################

print
print '*********************************************************************'
print '      SCRIPT FOR AUTOMATIC SEARCH OF VARIABLE STARS'
print '*********************************************************************'

#------------------------------------------------------------------------------------
#	selects images
#------------------------------------------------------------------------------------

#------ checking quality of measured frames ------

print
print '---------------------------------------------------------------------'
print '      Step 1 : checking the input images'
print '---------------------------------------------------------------------'
print
Imagelist2 = photometry.check_pictures_quality(Imagelist, outputname1)

#------ calculating shifts ------

print
print '---------------------------------------------------------------------'
print '      Step 2 : calculation of image shifts'
print '---------------------------------------------------------------------'
print
Imagelist3, shifts = photometry.calculate_shifts(Imagelist2, outputname2, detection_area = psf_fit_area, nb_stars = 12)

#------------------------------------------------------------------------------------
#		Calculating reference image
#------------------------------------------------------------------------------------

print
print '---------------------------------------------------------------------'
print '      Step 3 : calculating reference image and detecting stars'
print '---------------------------------------------------------------------'
print

im = astro.get_imagedata(Imagelist3[:10])
print
print "co-aligning images ..."
for i in range(10):
	im[i,:,:] = astro.translate_Bspline(im[i,:,:], [-shifts[i][0],-shifts[i][1]], 3)	
	print Imagelist3[i].split("\\")[-1]
print
print "stacking images ..."
imref = astro.add_sigma_clipping(im, 3.0, 1)
del im

ans = 'no'
while ans!='yes':
	while 1:
		try:
			print
			nbtarget = int(raw_input("how many stars do you want to measure ? Please enter a number:"))
			break
		except:
			print "please enter an integer !"

	#--- finding stars on first image ---
	xy_ref = astro.find_stars(imref, nbtarget, detection_area = max_detection_area, saturation = [True,conf.saturation])
	xy_ref = astro.centroid_stars(imref, xy_ref, 2)
	xy_shift = np.array(xy_ref)*0.

	#--- plotting detected stars ---
	fig, ax = astro.display(imref, title="Detected stars on reference image")

	astro.maximize_window(u"C:\\WINDOWS\\system32\\cmd.exe")
	sx, sy = astro.get_window_size(u"C:\\WINDOWS\\system32\\cmd.exe")
	fx, fy = astro.get_window_position("Detected stars on reference image")
	astro.move_window("Detected stars on reference image", [sx, fy])
	astro.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")
	
	ax.scatter(xy_ref[:,1],xy_ref[:,0],marker='+',color='g')
	plt.sca(ax)
	plt.xlim([0,imref.shape[1]])
	plt.ylim([0,imref.shape[0]])
	plt.draw()
	ans = raw_input('Is the number of detected stars OK (yes/no) ?')
	plt.close()

print
print '---------------------------------------------------------------------'
print '      Step 4 : aperture photometry'
print '---------------------------------------------------------------------'
print

dateobs = astro.julian_day(astro.get_headervalues(Imagelist3, 'DATE-OBS'))

xy = []
mag = []

def parallel_astrometry(i, Imagelist3, nbdisk, max_detection_area, shifts, xy_shift, xy_ref, thdist):
	image = aspylib.astro.get_imagedata([Imagelist3[i]])
	xy_image = aspylib.astro.find_stars(image, nbdisk, detection_area = max_detection_area, saturation = [True,aspylib.conf.saturation*0.9])
	xy_image = aspylib.astro.centroid_stars(image, xy_image, 2)
	xy_shift[:,0] = xy_ref[:,0] + shifts[i,0]
	xy_shift[:,1] = xy_ref[:,1] + shifts[i,1]	
	x,y = aspylib.astrometry.matching_pairs(xy_shift[:nbdisk,:],xy_image,thdist)

	coordinates = numpy.vstack([xy_shift[x,0], xy_shift[x,1], xy_image[y,0], xy_image[y,1]]).transpose()	
	errfunc = lambda p, xy: (aspylib.astrometry.distorsion_law_linear(p, xy[:,:2]) - xy[:,2:]).flatten() # Distance to the target function
	p0 = [0., 1., 0., 0., 0., 1.]
	p1, success = scipy.optimize.leastsq(errfunc, p0, args=(coordinates))
	print p1
	return aspylib.astrometry.distorsion_law_linear(p1,xy_shift)

def parallel_photometry(i, Imagelist3, sizes, xy):
	image = aspylib.astro.get_imagedata([Imagelist3[i]])
	return aspylib.photometry.aperture_photometry(image, sizes, xy, method = 'centroid', error_messages = 0)

# tuple of all parallel python servers to connect with
job_server = pp.Server()
print
print "Starting pp with", job_server.get_ncpus(), "workers"
t0 = time.time()
jobs = []
for i in range(len(Imagelist3)):	
	jobs.append(job_server.submit(parallel_astrometry,(i, Imagelist3, nbdisk, max_detection_area, shifts, xy_shift, xy_ref, thdist), (), ("scipy.optimize", "numpy", "aspylib.astro", "aspylib.conf", "aspylib.astrometry",)))

for job in jobs:
	out = job()
	xy.append(out)

xy = np.array(xy)
job_server.destroy()
print time.time()-t0,"seconds"

# tuple of all parallel python servers to connect with
job_server = pp.Server()
print
print "Starting pp with", job_server.get_ncpus(), "workers"
t0 = time.time()
jobs = []
for i in range(len(Imagelist3)):	
	jobs.append(job_server.submit(parallel_photometry,(i, Imagelist3, sizes, xy[i,:,:]), (), ("aspylib.astro", "aspylib.photometry",)))

for job in jobs:
	out = job()
	mag.append(out)
	
mag = np.array(mag)
job_server.destroy()
print time.time()-t0,"seconds"

#for i in range(len(Imagelist3)):	
#	mag.append(parallel_photometry(i, Imagelist3, sizes, xy[i,:,:]))
	
#*********************************************************************************************************
#		Calculating statistics
#*********************************************************************************************************

print
print '---------------------------------------------------------------------'
print '      Step 5 : processing results'
print '---------------------------------------------------------------------'
print

diff_mag, stats_diff, quality_flag, ref_stars, which_disk = photometry.compute_statistics(mag, nbref = nbref, plotting = True)

#*********************************************************************************************************
#		Exporting photometry results
#*********************************************************************************************************

print
print '---------------------------------------------------------------------'
print '      Step 6 : writing results in output file'
print '---------------------------------------------------------------------'
print

photometry.write_auto_magnitudes(Imagelist3, dateobs, xy, sizes, diff_mag, stats_diff, quality_flag, ref_stars, which_disk, outputname3)
#---> output file written only for first disk

raw_input("finished")
