# -*- coding: iso_8859_1 -*-

import numpy as np
import time, os, datetime, warnings
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
plt.ion()

#*********************************************************************************************************
#		Input parameters
#*********************************************************************************************************

#--- input frames ---
Folder = u"C:\\images\\"
File = Folder + 'target_' + "03_magnitudes.txt"

nb_stars = 1
mode = 1

#mode = 1 --> plot is magnitude / global noise
#mode = 2 --> plot is local noise / global noise
#*********************************************************************************************************
#		Functions to read files
#*********************************************************************************************************

def load_data(filename):

	#display file name
	file_path = filename.split('\\')
	file_core = file_path[len(file_path)-1]
	print "... loading data in %s ..." % file_core

	#open file and reads data
	f=open(filename, 'r')
	try:
		data = []
		flag = 0
		notlist = ["\n", "#"]
		for line in f:
			if flag:
				datalist = line.split('\t')
				if datalist[0] not in notlist:
					#stores data in array
					x = []
					for dd in datalist:
						if dd!='\n':
							if (dd[:8]=='1.#QNAN0')or(dd[:2]=='--')or(dd[:3]=='nan'):
								x.append(np.nan)
							else:
								x.append(float(dd))
					data.append(x)

			if (1-flag):
				if line[:15]=="#--- JULIAN DAY":
					flag = 1
	finally:
		f.close()
	return np.array(data)

def load_info(filename):

	#display file name
	file_path = filename.split('\\')
	file_core = file_path[len(file_path)-1]
	print "... loading info in %s ..." % file_core

	#open file and reads data
	f=open(filename, 'r')
	try:
		info = []
		flag = 0
		notlist = ["\n", "#"]
		for line in f:		

			if flag:
				if line[:15]=="#--- JULIAN DAY":
					flag = 0					
			
			if flag:
				datalist = line.split('\t')
				if datalist[0][0] not in notlist:
					#stores data in array
					x = []
					for i,dd in enumerate(datalist):
						if (i!=0)and(dd!='\n'):
							if (dd[:8]=='1.#QNAN0')or(dd[:2]=='--')or(dd[:3]=='nan'):
								x.append(float(np.nan))
							else:
								x.append(float(dd))
					info.append(x)
			
			if (1-flag):
				if line[:12]=="#--- OBJECTS":
					flag = 1
	finally:
		f.close()
	return np.array(info)

#*********************************************************************************************************
#		Read information
#*********************************************************************************************************

data = load_data(File)
print "number of images:",np.shape(data)[0]
print "measured objects:",np.shape(data)[1]-1
print

info = load_info(File)
quality_flag = info[6,:]

print "nb of objects with given quality:"
print "quality 0:",np.sum(info[6,:]==0)
print "quality 1:",np.sum(info[6,:]==1)
print "quality 2:",np.sum(info[6,:]==2)
print "quality 3:",np.sum(info[6,:]==3)
print "quality 4:",np.sum(info[6,:]==4)
print

ind0 = list(np.where(np.array(quality_flag)==0)[0])
ind1 = list(np.where(np.array(quality_flag)==1)[0])
ind2 = list(np.where(np.array(quality_flag)==2)[0])
ind3 = list(np.where(np.array(quality_flag)==3)[0])
ind4 = list(np.where(np.array(quality_flag)==4)[0])

flagplot = quality_flag <= 4

#*********************************************************************************************************
#		Plot information
#*********************************************************************************************************

#plt.figure()
#for i in ind0:
#	plt.plot(data[:,0],data[:,i+1])
#for i in ind1:
#	plt.plot(data[:,0],data[:,i+1])
#plt.grid()
#plt.draw()
#ax = plt.gca()
#ax.set_ylim(ax.get_ylim()[::-1])

xy = []

fig = plt.figure()
ax = plt.subplot(111)
if mode==1:
	if len(ind0)>0:
		ax.scatter(info[3,ind0],info[4,ind0],color='g', s=10, label='< 0.05')
	if len(ind1)>0:
		ax.scatter(info[3,ind1],info[4,ind1],color='#bbfe00', s=10, label='< 0.10')
	if len(ind2)>0:
		ax.scatter(info[3,ind2],info[4,ind2],color='y', s=10, label='< 0.25')
	if len(ind3)>0:
		ax.scatter(info[3,ind3],info[4,ind3],color='#fe8c00', s=10, label='< 0.50')
	if len(ind4)>0:
		ax.scatter(info[3,ind4],info[4,ind4],color='r', s=10, label='> 0.50')
	plt.xlabel('instrumental magnitude')
	plt.ylabel('global magnitude noise')
elif mode==2:
	if len(ind0)>0:
		ax.scatter(info[5,ind0],info[4,ind0],color='g', s=10, label='< 0.05')
	if len(ind1)>0:
		ax.scatter(info[5,ind1],info[4,ind1],color='#bbfe00', s=10, label='< 0.10')
	if len(ind2)>0:
		ax.scatter(info[5,ind2],info[4,ind2],color='y', s=10, label='< 0.25')
	if len(ind3)>0:
		ax.scatter(info[5,ind3],info[4,ind3],color='#fe8c00', s=10, label='< 0.50')
	if len(ind4)>0:
		ax.scatter(info[5,ind4],info[4,ind4],color='r', s=10, label='> 0.50')
	plt.xlabel('local magnitude noise')
	plt.ylabel('global magnitude noise')
plt.grid()
box = ax.get_position()
ax.set_position([box.x0, box.y0, box.width * 0.87, box.height])
leg = plt.legend(title = r'$\mathrm{\mathsf{max(|m_{i+1}-m_i|)}}$', prop=fm.FontProperties(size=11), loc='center left', bbox_to_anchor=(1.0, 0.5))
leg.get_title().set_fontproperties(fm.FontProperties(size=13))
plt.draw()

ax2 = []
warnings.simplefilter("ignore") 
def on_click(event): 
	global xy, data, info, ax2
	if event.inaxes is not None: 
		if event.button==3: 
			xy = [event.xdata,event.ydata]
			if mode==1:
				dist = np.sqrt((xy[0]-info[3,:]*flagplot)**2.+25.*(xy[1]-info[4,:]*flagplot)**2.)
			elif mode==2:
				dist = np.sqrt((xy[0]-info[5,:]*flagplot)**2.+(xy[1]-info[4,:]*flagplot)**2.)
			indices = np.argsort(dist)
			if isinstance(ax2,list):
				fig2 = plt.figure()
				ax2 = fig2.add_subplot(111)
			else:
				plt.sca(ax2)
				plt.cla()
			mini = []
			maxi = []
			print
			for i in range(nb_stars):
				plt.plot(data[:,0],data[:,indices[i]+1],label="object "+str(indices[i]+1))
				mask = ~(np.isnan(data[:,indices[i]+1]))
				mini.append(np.min(data[mask,indices[i]+1]))
				maxi.append(np.max(data[mask,indices[i]+1]))
				print "------ object",indices[i]+1,"------"
				print "AsPyLib (X,Y) position: %(x).3f, %(y).3f" % {'x':info[1,indices[i]],'y':info[2,indices[i]]}
				print "Prism (X,Y) position: %(x).3f, %(y).3f" % {'x':1+info[2,indices[i]],'y':1+info[1,indices[i]]}
				print "mean mag: %.5f" % info[3,indices[i]]
				print "local noise: %.5f" % info[4,indices[i]]
				print "global noise: %.5f" % info[5,indices[i]]
				print "quality flag: %.0f" % info[6,indices[i]]
			if max(maxi)-min(mini)<0.5:
				av = (max(maxi)+min(mini))*0.5
				plt.ylim([av-0.25,av+0.25])
			plt.grid()
			plt.legend()
			ax2.set_ylim(ax2.get_ylim()[::-1])
			plt.draw()
	else:
		fig.canvas.mpl_disconnect(cid) 
		fig.canvas.stop_event_loop_default() 		

cid = fig.canvas.mpl_connect('button_press_event', on_click) 
fig.canvas.start_event_loop_default() 
warnings.resetwarnings() 

raw_input("finished")