# -*- coding: iso_8859_1 -*-

import pyfits
import os, datetime
import numpy as np
import aspylib.conf as conf


def save_imagelist(images, headers, Imagelist):
	"""
	---------------------
	Purpose
	Saves a list of FITS images, given their data and header information.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* headers (list) = has the form [dictionary_1, dictionary_2, ..., dictionary_n]. Each list element dictionary_i is a Python dictionary that contains all the headers for the ith FITS image.
	* Imagelist (list) = has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image file.
	---------------------
	Output = no output
	---------------------
	"""
	
	#--- conditioning data ---
	if len(images.shape)==2:
		images = np.tile(images,(1,1,1))	

	#--- checking for saturation ---
	test = (images<conf.saturation-0.001)
	images = (1.-test)*conf.saturation + test*images		
	
	#--- saturation value + cutting image values ---	
	bzerovalue = round(conf.saturation-32767.)
	if bzerovalue==0:
		images = np.array(np.round(images,0),dtype=np.int16)		#in principle not required for bzerovalue!=0 due to hdu.scale instruction
	
	for i in range(images.shape[0]):
		#--- create FITS ---
		hdu = pyfits.PrimaryHDU()
		hdu.data = np.array(images[i,:,:])

		#managing FITS data offset and scale
		if bzerovalue!=0:
			hdu.scale('int16','', bzero=bzerovalue)					

		del hdu.header['BZERO']
		del hdu.header['BSCALE']
		hdu.header.update("BZERO",bzerovalue,"")
		hdu.header.update("BSCALE",1.,"")

		#adding date of file writing
		now = datetime.datetime.now()
		date = now.strftime("%Y-%m-%dT%H:%M:%S")
		hdu.header.update("DATE",str(date),'Date of file writing')

		#writing all other headers
		for hh in sorted(headers[i].iterkeys()):
			if (hh!='BZERO')and(hh!='BSCALE')and(hh!='DATE')and(hh!='SIMPLE')and(hh!='BITPIX')and(hh!='NAXIS')and(hh!='NAXIS1')and(hh!='NAXIS2')and(hh!='EXTEND'):
				hdu.header.update(hh,headers[i][hh][0],headers[i][hh][1])
			
		#--- save FITS ---
		if os.path.exists(Imagelist[i]):
			os.remove(Imagelist[i])
		hdu.writeto(Imagelist[i])
	#-------------------------------------

