# -*- coding: iso_8859_1 -*-
"""
	AsPyLib library version 1.3.1
    Copyright (C) 2012 Jerome Caron 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
#------------------------------------------------------------------------------------------------------------
#		aspylib.conf = module where aspylib settings are defined
#------------------------------------------------------------------------------------------------------------

import numpy as np

#------------------------------------------------------------------------------------------
#	constants
#------------------------------------------------------------------------------------------

saturation = 32767.		#is updated when an image is loaded

#--- updating constants --
def update_saturation(newsat):
	global saturation
	saturation = newsat
	return 0

#------------------------------------------------------------------------------------------
#	default headers
#------------------------------------------------------------------------------------------

def set_headers(Config):

	#depending on Config, the Python dictionary hd will contain different headers
	# - headers like 'BINX':[] mean that the header will be taken from an existing image when calling astro.create_headers
	# - headers like 'CAMERA':['Audine','Data acquisition camera'] will be directly used like this when calling astro.create_headers

	hd = {}
	
	if Config == 0: 	#just leave following headers unchanged

		hd = {'BINX':[],'BINY':[],'BIN1':[],'BIN2':[],'DATE-OBS':[],'DATE-END':[],'EXPTIME':[],'EXPOSURE':[],'GPS-DATE':[],'OBJECT':[]}

	if Config == 1: 	#Audine KAF0400 + Visac with focal reducer

		hd = {'BINX':[],'BINY':[],'BIN1':[],'BIN2':[],'DATE-OBS':[],'DATE-END':[],'EXPTIME':[],'EXPOSURE':[],'GPS-DATE':[],'OBJECT':[]}
	
		hd.update({'OBSERVER':	['Jerome CARON','Observer name']})
		hd.update({'ORIGIN':		['Uiterstegracht, Leiden, Netherlands','Origin place of FITS file']})
		hd.update({'SITELAT':		['52:09:26.6 North','Latitude where data acquisition was made']})
		hd.update({'SITELONG':	['04:29:51.9 East','Longitude where data acquisition was made']})

		hd.update({'CAMERA':		['Audine Kaf0402-ME','Data acquisition camera']})

		hd.update({'TELESCOP':	['Vixen VISAC VC200L + focal reducer','Data acquisition telescope']})
		hd.update({'DIAMETER':	[200.,'Telescope aperture diameter (mm)']})
		hd.update({'FOCAL':		[1280.,'Telescope focal length (mm)']})

	return hd
		