# -*- coding: iso_8859_1 -*-

import os, datetime
import numpy as np
from aspylib import astro

def write_auto_magnitudes(Imagelist, dateobs, xy, sizes, mag, stats, quality_flag, ref_stars, which_disk, outputname):
	
	if os.path.exists(outputname):
		os.remove(outputname)
	f = open(outputname, 'w')

	#--- nb of stars to be written to file ---
	ind = list(np.where(np.array(quality_flag)>=0)[0])
	
	#--- file header ---
	f.write('#--- MAIN INFORMATION ------------------\n')
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write(str(len(Imagelist))+' images processed\n')
	f.write(str(len(quality_flag))+' objects detected\n')
	f.write(str(len(ind))+' objects successfully measured\n')
	f.write('first image = '+(Imagelist[0]).encode('iso_8859_1')+'\n')
	f.write('datation at start of exposure\n')
	h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
	h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
	if h1!='':
		f.write('exposure time (sec):\t'+str(int(np.round(h1)))+'\n')
	elif h2!='':
		f.write('exposure time (sec):\t'+str(int(np.round(h2)))+'\n')
	f.write('photometry with apertures:\n')
	for i in range(len(sizes)):	
		f.write('disk_'+str(i)+'\t'+str(sizes[i][0])+' - '+str(sizes[i][1])+' - '+str(sizes[i][2])+'\n')
	f.write('\n')

	#--- reference stars ---
	#print ref_stars
	f.write('#--- REFERENCE STARS --\n')
	for i in range(len(sizes)):	
		f.write('disk_'+str(i)+':\n')
		f.write('nb of ref stars:' + str(len(ref_stars[i,:])) + '\n')
		f.write('ref stars:')
		for ref in np.sort(ref_stars[i,:]):
			f.write('\t' + str(ref+1))
		f.write('\n')
	f.write('\n')

	#--- stars X,Y coordinates ---
	f.write('#--- OBJECTS COORDINATES ON FIRST FRAME --\n')
	f.write('Object nb')
	for j in range(len(ind)):
		f.write('\t'+str(j+1))
	f.write('\n')
	f.write('X position')
	for j in ind:
		f.write('\t'+"%.4f"%xy[0][j][0])
	f.write('\n')
	f.write('Y position')
	for j in ind:
		f.write('\t'+"%.4f"%xy[0][j][1])
	f.write('\n\n')

	#--- statistics about photometry ---
	signal, noise_global, noise_local = stats

	f.write('#--- STATISTICS ------------------------\n')
	f.write('mag signal')
	for j in ind:
		f.write('\t'+"%.6f"%signal[j])
	f.write('\n')
	f.write('global noise')
	for j in ind:
		f.write('\t'+"%.6f"%noise_global[j])
	f.write('\n')
	f.write('local noise')
	for j in ind:
		f.write('\t'+"%.6f"%noise_local[j])
	f.write('\n')
	f.write('quality flag')
	for j in ind:
		f.write('\t'+str(quality_flag[j]))
	f.write('\n')
	f.write('disk used for photometry')
	for j in ind:
		f.write('\t'+str(which_disk[j]))
	f.write('\n\n')
	
	#--- measured magnitudes ---
	f.write('#--- JULIAN DAY + MEASURED MAGNITUDES ---------------\n')
	data = np.ndarray((len(dateobs),len(ind)+1))
	data[:,0] = np.array(dateobs)
	data[:,1:] = np.ma.filled(mag[:,ind],fill_value=np.nan)
	np.savetxt(f,data,delimiter='\t',fmt='%.6f')

	f.close()
	#-------------------------------------