# -*- coding: iso_8859_1 -*-

from aspylib import astro, conf
import time, sys
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages

def select_stars(Imagelist, shifts, outputname, detection_area = 10, method = 'centroid'):
		
	#------------------------------------------------------------------------------------
	#	loading basic information
	#------------------------------------------------------------------------------------

	#------ initialisation of basic vbles ------
	xy_mov = []
	xy_vbl = []
	xy_ref = []
	
	#------ loading images ------
	image1 = astro.get_imagedata([Imagelist[0]], text_display=False)
	image2 = astro.get_imagedata([Imagelist[-1]], text_display=False)

	#------ setting up console windows ------
	astro.maximize_window(u"C:\\WINDOWS\\system32\\cmd.exe")
	sx, sy = astro.get_window_size(u"C:\\WINDOWS\\system32\\cmd.exe")
	
	#------------------------------------------------------------------------------------
	#	selecting MOVING OBJECTS
	#------------------------------------------------------------------------------------

	print
	print '--- OBJECTS FOR photometry ------------------------------------------'
	print
	print 'How many MOVING OBJECTS do you want to measure ?'
	fig, ax = astro.display(image1, title="First image")
	fx, fy = astro.get_window_position("First image")
	astro.move_window("First image", [sx, fy])
	astro.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")
	while 1:
		try:
			nb = np.int(raw_input('nb = '))
			break
		except:
			print "please enter an integer !"
		
	#picks up axes with original scale, for saving figure as .pdf in the end
	limits = ax.axis()

	if nb!=0:
		print
		print 'First image : right-click on exact positions of the objects'
		fig2, ax2 = astro.display(image2, title="Last image")
		fx, fy = astro.get_window_position("Last image")
		astro.move_window("Last image", [sx, fy])
		astro.set_focus("First image")
		xy1 = astro.select(fig,ax,nb,color='r')
		
		print
		print 'Last image : right-click on exact positions of the objects (same order)'
		astro.set_focus("Last image")
		xy2 = astro.select(fig2,ax2,nb,color='r')		
		plt.close()

		for i in range(len(xy1)):
			if (xy1[i][0]-detection_area<0)or(xy1[i][0]+detection_area+1>image1.shape[0])or(xy1[i][1]-detection_area<0)or(xy1[i][1]+detection_area+1>image1.shape[1]):
				print 'moving object',i+1,'is too close from border of first image'
				print 'this object is removed'
			elif (xy2[i][0]-detection_area<0)or(xy2[i][0]+detection_area+1>image1.shape[0])or(xy2[i][1]-detection_area<0)or(xy2[i][1]+detection_area+1>image1.shape[1]):
				print 'moving object',i+1,'is too close from border of last image'
				print 'this object is removed'
			else:
				xy_mov.append(xy1[i] + xy2[i])
				
	#------------------------------------------------------------------------------------
	#	selecting VARIABLE STARS
	#------------------------------------------------------------------------------------
	ax.axis(limits)
	plt.draw()
	
	print
	print 'How many VARIABLE STARS do you want to measure ?'
	astro.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")
	while 1:
		try:
			nb = np.int(raw_input('nb = '))
			break
		except:
			print "please enter an integer !"
	
	if nb!=0:
		print
		print 'First image : right-click on exact positions of stars'
		astro.set_focus("First image")
		xy1 = astro.select(fig,ax,nb,color='m')

		for i in range(len(xy1)):
			if (xy1[i][0]-detection_area<0)or(xy1[i][0]+detection_area+1>image1.shape[0])or(xy1[i][1]-detection_area<0)or(xy1[i][1]+detection_area+1>image1.shape[1]):
				print 'variable star',i+1,'is too close from border of first image'
				print 'this star is removed'
			else:
				xy_vbl.append(xy1[i])

	#------------------------------------------------------------------------------------
	#	selecting REFERENCE STARS
	#------------------------------------------------------------------------------------
	ax.axis(limits)
	plt.draw()

	print
	print
	print '--- REFERENCE STARS -------------------------------------------------'
	print
	print 'How many REFERENCE STARS do you want to measure ?'
	astro.set_focus(u"C:\\WINDOWS\\System32\\cmd.exe")
	while 1:
		try:
			nb = np.int(raw_input('nb = '))
			break
		except:
			print "please enter an integer !"
	
	if nb!=0:
		print
		print 'First image : right-click on exact positions of stars'
		astro.set_focus("First image")
		xy1 = astro.select(fig,ax,nb,color='g')
		
		for i in range(len(xy1)):
			if (xy1[i][0]-detection_area<0)or(xy1[i][0]+detection_area+1>image1.shape[0])or(xy1[i][1]-detection_area<0)or(xy1[i][1]+detection_area+1>image1.shape[1]):
				print 'reference star',i+1,'is too close from border of first image'
				print 'this star is removed'
			else:
				xy_ref.append(xy1[i])
	
	#------------------------------------------------------------------------------------
	#	saving selected stars in .pdf
	#------------------------------------------------------------------------------------

	ax.axis(limits)
	pp = PdfPages(outputname)
	pp.savefig()
	pp.close()
	plt.close()
	
	#------------------------------------------------------------------------------------
	#	check that one object + one ref star at least have been selected
	#------------------------------------------------------------------------------------

	nb_obj = len(xy_mov)+len(xy_vbl)
	nb_ref = len(xy_ref)

	if nb_obj==0:
		print
		raw_input('no object has been selected, press ENTER to close window')
		sys.exit()

	if nb_ref==0:
		print
		raw_input('no reference star has been selected, press ENTER to close window')
		sys.exit()

	#------------------------------------------------------------------------------------
	#	loading basic information (2)
	#------------------------------------------------------------------------------------

	#--- initialising main variables ---
	time_format = "%Y-%m-%dT%H:%M:%S"
	xy=[]		#X-Y positions on 1st image
	dxy=[]		#X-Y positions change per unit time
	names = []

	#------ first image ------
	date1 = astro.get_headervalues([Imagelist[0]], 'DATE-OBS')
	t1 = time.mktime(time.strptime(date1[0].split('.')[0],time_format))
	
	#------ last image ------
	date2 = astro.get_headervalues([Imagelist[-1]], 'DATE-OBS')
	t2 = time.mktime(time.strptime(date2[0].split('.')[0],time_format))

	#------------------------------------------------------------------------------------
	#	calculating approximate coordinates on each frame
	#------------------------------------------------------------------------------------	
	
	if len(xy_mov)!=0:		
		xs, ys = shifts[-1]		#shift between first and last images

		for i in range(len(xy_mov)):
				
			#--- collecting information ---
			xy.append([np.float(xy_mov[i][0]), np.float(xy_mov[i][1])])
			dxy.append([np.float(xy_mov[i][2]-xy_mov[i][0]-xs)/(t2-t1), np.float(xy_mov[i][3]-xy_mov[i][1]-ys)/(t2-t1)])
			names.append('moving_object_'+str(i+1))

	if len(xy_vbl)!=0:		
		for i in range(len(xy_vbl)):
			xy.append([np.float(xy_vbl[i][0]), np.float(xy_vbl[i][1])])
			dxy.append([0., 0.])
			names.append('vble_star_'+str(i+1))

	if len(xy_ref)!=0:		
		for i in range(len(xy_ref)):
			xy.append([np.float(xy_ref[i][0]), np.float(xy_ref[i][1])])
			dxy.append([0., 0.])
			names.append('ref_star_'+str(i+1))

	xy = np.array(xy)
	dxy = np.array(dxy)
			
	#------------------------------------------------------------------------------------
	#	calculating positions of stars on all frames
	#------------------------------------------------------------------------------------

	if method=='select':
		print
		print '--- CALCULATING STAR POSITIONS --------------------------------------'
		print

		xy_sel = []
		for i in range(len(Imagelist)):					#loop on images
		
			print 'calculating star position on :',Imagelist[i].split('\\')[-1]
			if i==0:
				image = image1
			else:		
				image = astro.get_imagedata([Imagelist[i]], text_display=False)

			date = astro.get_headervalues([Imagelist[i]], 'DATE-OBS')
			t = time.mktime(time.strptime(date[0].split('.')[0],time_format))		
			xyi = np.round(xy+shifts[i]+(t-t1)*dxy)

			xy_sel.append(xyi)
		xy_sel = np.array(xy_sel)
		return names, xy_sel
	
	elif method=='maximum':
		print
		print '--- CALCULATING LOCAL MAX FOR STAR POSITION -------------------------'
		print

		xy_max = []
		for i in range(len(Imagelist)):					#loop on images
		
			print 'calculating local maxima on :',Imagelist[i].split('\\')[-1]
			if i==0:
				image = image1
			else:		
				image = astro.get_imagedata([Imagelist[i]], text_display=False)

			date = astro.get_headervalues([Imagelist[i]], 'DATE-OBS')
			t = time.mktime(time.strptime(date[0].split('.')[0],time_format))		
			xyi = np.round(xy+shifts[i]+(t-t1)*dxy)

			xy_image = astro.max_stars(image, xyi, detection_area, saturation = [True,conf.saturation], error_messages = 2)			
			xy_max.append(xy_image)
		xy_max = np.array(xy_max)
		return names, xy_max
		
	elif method=='centroid':
		print
		print '--- CALCULATING STAR CENTROID WITH PSF FIT --------------------------'
		print

		xy_max = []
		for i in range(len(Imagelist)):					#loop on images
		
			print 'calculating local maxima on :',Imagelist[i].split('\\')[-1]
			if i==0:
				image = image1
			else:		
				image = astro.get_imagedata([Imagelist[i]], text_display=False)

			date = astro.get_headervalues([Imagelist[i]], 'DATE-OBS')
			t = time.mktime(time.strptime(date[0].split('.')[0],time_format))		
			xyi = np.round(xy+shifts[i]+(t-t1)*dxy)

			xy_image = astro.max_stars(image, xyi, detection_area, saturation = [True,conf.saturation], error_messages = 2)			
			xy_max.append(xy_image)
		xy_max = np.array(xy_max)
		#-----------------------------------------------------------------------
		xy_psf = []
		print	
		for i in range(len(Imagelist)):					#loop on images
			
			print 'PSF fitting on :',Imagelist[i].split('\\')[-1]
			if i==0:
				image = image1
			else:
				image = astro.get_imagedata([Imagelist[i]], text_display=False)

			xy_image = astro.fit_stars(image, xy_max[i], detection_area, maxshift = [True,detection_area*0.5], error_messages = 2)
		
			xy_psf.append(xy_image)
		xy_psf = np.array(xy_psf)
			
		return names, xy_psf
