# -*- coding: iso_8859_1 -*-

import numpy as np
import os, datetime
from aspylib import astro

def write_best_magnitude(outputname, Imagelist, names, obj_index, sizes, ref_1, mag_1, sig_1, dateobs, nb=10, rejdata = [], rejsizes = [], rejstar = []):

	nb_obj = len([i for i in names if (i[0]=='m')or(i[0]=='v')])  
	nb_ref = len([i for i in names if (i[0]=='r')])  
	nb_results = np.int(min([nb,len(sizes)*((2.**nb_ref)-1.)]))

	#---------- writing results to output file -------------	
	filename = outputname.split('.')[0] + '_' + names[obj_index] + '.' + outputname.split('.')[1]
	if os.path.exists(filename):
		os.remove(filename)

	f = open(filename, 'w')

	#--- file header ---
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write("Working directory:\n"+(os.path.split(Imagelist[0])[0]).encode('iso_8859_1')+"\n\n")

	f.write(str(len(Imagelist))+' images processed\n')
	f.write('first processed image = '+Imagelist[0].split('\\')[-1]+'\n\n')

	if len(rejdata)>0:
		f.write('The following image(s) have been rejected:\n')
		for i in rejdata:
			f.write(Imagelist[i].split('\\')[-1]+'\n')
		f.write("\n")			

	if len(rejsizes)>0:
		f.write('Disk sizes rejected:\n')
		for disk in rejsizes:
			f.write(str(sizes[disk])+'\n')
		f.write('\n')
	
	if len(rejstar)>0:
		f.write('Reference stars rejected:\n')
		for star in rejstar:
			f.write(str(names[star])+'\n')
		f.write('\n')
	
	f.write('datation at start of exposure\n')
	h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
	h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
	if h1!='':
		f.write('exposure time (sec):\t'+str(int(np.round(h1)))+'\n\n')
	elif h2!='':
		f.write('exposure time (sec):\t'+str(int(np.round(h2)))+'\n\n')

	#--- measures magnitudes ---
	f.write("############### " + names[obj_index] + " ###############\n")
			
	#------ 1st line ------
	f.write("solutions")
	for i in range(nb_results):
		f.write("\t"+str(i+1))
	f.write("\n")
	
	#------ 2nd line ------
	f.write("apertures")
	for i in range(nb_results):
		f.write("\t"+str(sizes[ref_1[i][0]]))
	f.write("\n")

	#------ 3rd line ------
	f.write("reference stars")
	for i in range(nb_results):
		f.write("\t")
		f.write(str(ref_1[i][1][0]))	
		for j in range(len(ref_1[i][1])-1):		#loop on reference stars for selected method
			f.write('+'+str(ref_1[i][1][j+1]))
	f.write("\n")
	
	#------ 4th line ------
	f.write("magnitude noise")
	for i in range(nb_results):
		f.write("\t"+"%.6f"%sig_1[i])
	f.write("\n")

	#------ magnitudes ------
	f.write('Julian day')
	for i in range(nb_results):
		f.write('\tmag')
	f.write('\n')

	dateobs_rej = [dateobs[j] for j in range(len(Imagelist)) if j not in rejdata]
	for j in range(len(Imagelist)-len(rejdata)):
		f.write("%.6f"%dateobs_rej[j])
		for i in range(nb_results):
			f.write('\t'+"%.6f"%mag_1[i][j])
		f.write('\n')

	f.close()
	#-------------------------------------
